import dotenv from 'dotenv';
import { filter, Observable, Observer, Subscription, takeWhile } from 'rxjs';
import { v4 as uuidv4 } from 'uuid'
import ConsoleLogger from '../utils/log.utils';
import { WrappedMessage } from '../utils/message.ordering';
import { AdapterBase } from '../base/adapter.base';
import { AdapterEvent, FisMessage, Transport, TransportEvent, TransportMessage, TransportService } from '../interface/interface';

dotenv.config();
/* This transport manager will be instantiating the necessary transport to deal with tranmission and receiving from different receivers
So how?: */
export class ReceiverAdapter extends AdapterBase {
    private console!: ConsoleLogger

    constructor(adapterId: string, adapterType: Transport, transportService: TransportService) {
        super()
        
        this.console = new ConsoleLogger(`${adapterType}ReceiverAdapter`, ['adapter'])
        this.setAdapterProfile(adapterId, adapterType, 'Receiver')
        this.transportService = transportService
    }

    subscribe(): Observable<AdapterEvent> {
        this.console.log({ message: `Connector getting message bus for this connector: ${this.adapterID}` })
        return new Observable((observable: Observer<AdapterEvent>) => {
            const subscription: Subscription = this.transportService.subscribe().pipe(
                filter((message: TransportEvent) => message.event === 'New Message'),
                // take message only specific for this adapter. Although that itself wouldn't be necessary, considerng everything goes through transportEvent. I guess it's for better management
                filter((message: TransportEvent) => (message.data as TransportMessage).target == this.adapterID),
            ).subscribe((message: TransportEvent) => {
                this.console.log({ message: `Received ${(((message.data as TransportMessage).payload as WrappedMessage).payload as FisMessage).header.messageID} from ${((message.data as TransportMessage).target)}`, details: message })
                observable.next({
                    id: uuidv4(),
                    type: `Adapter Event`,
                    event: `New Message`,
                    date: new Date(),

                } as AdapterEvent);
            });

            // Clean up on unsubscription
            return () => {
                subscription.unsubscribe();
            };

        });
    }

}


